@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2006 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -12,11 +12,18 @@ Module defining common data to be used by all modules.
 #
 
 from __future__ import unicode_literals
+try:
+    str = unicode       # __IGNORE_EXCEPTION__
+except NameError:
+    pass
 
 import sys
 import os
+import re
+import shutil
 
-from PyQt5.QtCore import QDir, QLibraryInfo, QByteArray
+from PyQt5.QtCore import QDir, QLibraryInfo, QByteArray, QCoreApplication, \
+    QT_VERSION_STR, QT_VERSION, QProcess, qVersion
 
 # names of the various settings objects
 settingsNameOrganization = "Eric6"
@@ -27,7 +34,10 @@ settingsNameRecent = "eric6recent"
 recentNameMultiProject = "MultiProjects"
 recentNameProject = "Projects"
 recentNameFiles = "Files"
+recentNameHexFiles = "HexFiles"
 recentNameHosts = "Hosts6"
+recentNameBreakpointFiles = "BreakPointFiles"
+recentNameBreakpointConditions = "BreakPointConditions"
 
 configDir = None
 
@@ -38,7 +48,7 @@ def isWindowsPlatform():
     
     @return flag indicating Windows platform (boolean)
     """
-    return sys.platform.startswith("win")
+    return sys.platform.startswith(("win", "cygwin"))
 
 
 def isMacPlatform():
@@ -59,6 +69,32 @@ def isLinuxPlatform():
     return sys.platform.startswith("linux")
 
 
+def desktopName():
+    """
+    Function to determine the name of the desktop environment used
+    (Linux only).
+    
+    @return name of the desktop environment
+    @rtype str
+    """
+    if not isLinuxPlatform():
+        return ""
+    
+    currDesktop = os.environ.get("XDG_CURRENT_DESKTOP", "")
+    if currDesktop:
+        return currDesktop
+    
+    currDesktop = os.environ.get("XDG_SESSION_DESKTOP", "")
+    if currDesktop:
+        return currDesktop
+    
+    currDesktop = os.environ.get("GDMSESSION", "")
+    if currDesktop:
+        return currDesktop
+    
+    return ""
+
+
 def checkBlacklistedVersions():
     """
     Module functions to check for blacklisted versions of the prerequisites.
@@ -77,8 +113,11 @@ def checkBlacklistedVersions():
     
     # check version of sip
     try:
-        import sipconfig
-        sipVersion = sipconfig.Configuration().sip_version_str
+        try:
+            from PyQt5 import sip
+        except ImportError:
+            import sip
+        sipVersion = sip.SIP_VERSION_STR
         # always assume, that snapshots are good
         if "snapshot" not in sipVersion:
             # check for blacklisted versions
@@ -89,7 +128,7 @@ def checkBlacklistedVersions():
                         .format(vers))
                     print('Please install another version.')
                     return False
-    except ImportError:
+    except (ImportError, AttributeError):
         pass
     
     # check version of PyQt
@@ -133,16 +172,23 @@ def getConfigDir():
     if configDir is not None and os.path.exists(configDir):
         hp = configDir
     else:
+        cdn = ".eric6"
         if isWindowsPlatform():
-            cdn = "_eric6"
-        else:
-            cdn = ".eric6"
-            
-        hp = QDir.homePath()
-        dn = QDir(hp)
-        dn.mkdir(cdn)
-        hp += "/" + cdn
-    return QDir.toNativeSeparators(hp)
+            # migrate the old config directory (< v18.06)
+            cdnOld = "_eric6"
+            hpOld = os.path.join(os.path.expanduser("~"), cdnOld)
+            if os.path.exists(hpOld):
+                hpNew = os.path.join(os.path.expanduser("~"), cdn)
+                if os.path.exists(hpNew):
+                    # simply delete the old config directory
+                    shutil.rmtree(hpOld, True)
+                else:
+                    os.rename(hpOld, hpNew)
+        
+        hp = os.path.join(os.path.expanduser("~"), cdn)
+        if not os.path.exists(hp):
+            os.mkdir(hp)
+    return hp
 
 
 def setConfigDir(d):
@@ -180,22 +226,81 @@ def getPyQt5ModulesDirectory():
     return ""
     
 
+def getPyQtToolsPath(version=5):
+    """
+    Module function to get the path of the PyQt tools.
+    
+    @param version PyQt major version
+    @type int
+    @return path to the PyQt tools
+    @rtype str
+    """
+    import Preferences
+    
+    path = ""
+    
+    # step 1: check, if the user has configured a tools path
+    path = Preferences.getQt("PyQtToolsDir")
+    
+    # step 2: determine from used Python interpreter (pyrcc is test object)
+    if not path:
+        program = "pyrcc{0}".format(version)
+        if isWindowsPlatform():
+            program += ".exe"
+            dirName = os.path.dirname(sys.executable)
+            if os.path.exists(os.path.join(dirName, program)):
+                path = dirName
+            elif os.path.exists(os.path.join(dirName, "Scripts", program)):
+                path = os.path.join(dirName, "Scripts")
+        else:
+            dirName = os.path.dirname(sys.executable)
+            if os.path.exists(os.path.join(dirName, program)):
+                path = dirName
+    
+    return path
+
+
 def getQtBinariesPath():
     """
     Module function to get the path of the Qt binaries.
     
     @return path of the Qt binaries (string)
     """
+    import Preferences
+    
     path = ""
-    if isWindowsPlatform():
-        # check for PyQt5 installer first (designer is test object)
+    
+    # step 1: check, if the user has configured a tools path
+    path = Preferences.getQt("QtToolsDir")
+    
+    if not path and isWindowsPlatform():
+        # step 2.1: check for PyQt5 Windows installer (designer is test object)
         modDir = getPyQt5ModulesDirectory()
         if os.path.exists(os.path.join(modDir, "bin", "designer.exe")):
             path = os.path.join(modDir, "bin")
         elif os.path.exists(os.path.join(modDir, "designer.exe")):
             path = modDir
+        
+        if not path:
+            import distutils.sysconfig
+            # step 2.2.1: check for the pyqt5-tools wheel (new variant)
+            # (Windows only)
+            pyqt5ToolsPath = os.path.join(
+                distutils.sysconfig.get_python_lib(True), "pyqt5_tools")
+            if os.path.exists(os.path.join(pyqt5ToolsPath, "designer.exe")):
+                path = pyqt5ToolsPath
+            if not path:
+                # step 2.2.2: check for the pyqt5-tools wheel (old variant)
+                # (Windows only)
+                pyqt5ToolsPath = os.path.join(
+                    distutils.sysconfig.get_python_lib(True), "pyqt5-tools")
+                if os.path.exists(os.path.join(pyqt5ToolsPath,
+                                               "designer.exe")):
+                    path = pyqt5ToolsPath
     
     if not path:
+        # step 3: get the path from Qt
+        # Note: no Qt tools are to be found there for PyQt 5.7.0
         path = QLibraryInfo.location(QLibraryInfo.BinariesPath)
         if not os.path.exists(path):
             path = ""
@@ -203,48 +308,108 @@ def getQtBinariesPath():
     return QDir.toNativeSeparators(path)
 
 
+def translate(*args):
+    """
+    Module function to handle different PyQt 4/5 QCoreApplication.translate
+    parameter.
+    
+    @param args tuple of arguments from QCoreApplication.translate (tuple)
+    @return translated string (string)
+    """
+    if QT_VERSION_STR.startswith('4.'):
+        args = list(args)
+        args.insert(3, QCoreApplication.CodecForTr)
+    return QCoreApplication.translate(*args)
+
+
 ###############################################################################
-## functions for searching a Python2/3 interpreter
+## functions for version handling
 ###############################################################################
 
 
-def findPythonInterpreters(pyVersion):
+def versionToTuple(version):
     """
-    Module function for searching a Python interpreter.
+    Module function to convert a version string into a tuple.
     
-    @param pyVersion major Python version
-    @return list of interpreters found (list of strings)
+    Note: A version string consists of non-negative decimals separated by "."
+    optionally followed by a suffix. Suffix is everything after the last
+    decimal.
+    
+    @param version version string
+    @type str
+    @return version tuple without the suffix
+    @rtype tuple of int
     """
-    if pyVersion == 2:
-        winPathList = ["C:\\Python25", "C:\\Python26",
-                       "C:\\Python27", "C:\\Python28"]
-        posixVersionsList = ["2.5", "2.6", "2.7", "2.8"]
-    else:
-        winPathList = ["C:\\Python3{0}".format(x) for x in range(5)]
-        posixVersionsList = ["3.{0}".format(x) for x in range(5)]
-    posixPathList = ["/usr/bin", "/usr/local/bin"]
+    versionParts = []
     
-    interpreters = []
-    if isWindowsPlatform():
-        # search the interpreters on Windows platforms
-        for path in winPathList:
-            exeList = [
-                "python.exe",
-                "python{0}.{1}.exe".format(path[-2], path[-1]),
-            ]
-            for exe in exeList:
-                interpreter = os.path.join(path, exe)
-                if os.path.isfile(interpreter):
-                    interpreters.append(interpreter)
-    else:
-        # search interpreters on Posix and Mac platforms
-        for path in posixPathList:
-            for version in posixVersionsList:
-                interpreter = os.path.join(path, "python{0}".format(version))
-                if os.path.isfile(interpreter):
-                    interpreters.append(interpreter)
+    # step 1: extract suffix
+    version = re.split(r"[^\d.]", version)[0]
+    for part in version.split("."):
+        versionParts.append(int(part))
     
-    return interpreters
+    return tuple(versionParts)
+
+
+def qVersionTuple():
+    """
+    Module function to get the Qt version as a tuple.
+    
+    @return Qt version as a tuple
+    @rtype tuple of int
+    """
+    return (
+        (QT_VERSION & 0xff0000) >> 16,
+        (QT_VERSION & 0xff00) >> 8,
+        QT_VERSION & 0xff,
+    )
+
+
+###############################################################################
+## functions for extended string handling
+###############################################################################
+
+
+def strGroup(txt, sep, groupLen=4):
+    """
+    Module function to group a string into sub-strings separated by a
+    separator.
+    
+    @param txt text to be grouped
+    @type str
+    @param sep separator string
+    @type str
+    @param groupLen length of each group
+    @type int
+    @return result string
+    @rtype str
+    """
+    groups = []
+    
+    while len(txt) // groupLen != 0:
+        groups.insert(0, txt[-groupLen:])
+        txt = txt[:-groupLen]
+    if len(txt) > 0:
+        groups.insert(0, txt)
+    return sep.join(groups)
+
+
+def strToQByteArray(txt):
+    """
+    Module function to convert a Python string into a QByteArray.
+    
+    @param txt Python string to be converted
+    @type str, bytes, bytearray, unicode
+    @return converted QByteArray
+    @rtype QByteArray
+    """
+    if sys.version_info[0] == 2:
+        if isinstance(txt, unicode):    # __IGNORE_WARNING__
+            txt = txt.encode("utf-8")
+    else:
+        if isinstance(txt, str):
+            txt = txt.encode("utf-8")
+    
+    return QByteArray(txt)
 
 
 ###############################################################################
@@ -306,3 +471,30 @@ def toDict(value):
         return {}
     else:
         return value
+
+
+###############################################################################
+## functions for web browser variant detection
+###############################################################################
+
+
+def getWebBrowserSupport():
+    """
+    Module function to determine the supported web browser variant.
+    
+    @return string indicating the supported web browser variant ("QtWebEngine",
+        "QtWebKit" or "None")
+    @rtype str
+    """
+    from eric6config import getConfig
+    scriptPath = os.path.join(getConfig("ericDir"), "Tools",
+                              "webBrowserSupport.py")
+    proc = QProcess()
+    proc.start(sys.executable, [scriptPath, qVersion()])
+    if proc.waitForFinished(10000):
+        variant = str(proc.readAllStandardOutput(), "utf-8", 'replace').strip()
+    else:
+        variant = "None"
+    return variant
+#
+# eflag: noqa = M801
