@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2011 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2011 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -127,17 +127,20 @@ class VirusTotalAPI(QObject):
         import Helpviewer.HelpWindow
         nam = Helpviewer.HelpWindow.HelpWindow.networkAccessManager()
         reply = nam.post(request, params)
-        reply.finished.connect(self.__checkServiceKeyValidityFinished)
+        reply.finished.connect(
+            lambda: self.__checkServiceKeyValidityFinished(reply))
         self.__replies.append(reply)
     
-    def __checkServiceKeyValidityFinished(self):
+    def __checkServiceKeyValidityFinished(self, reply):
         """
         Private slot to determine the result of the service key validity check.
+        
+        @param reply reference to the network reply
+        @type QNetworkReply
         """
         res = False
         msg = ""
         
-        reply = self.sender()
         if reply.error() == QNetworkReply.NoError:
             res = True
         elif reply.error() == self.ServiceCode_InvalidKey:
@@ -168,11 +171,13 @@ class VirusTotalAPI(QObject):
         reply.finished.connect(self.__submitUrlFinished)
         self.__replies.append(reply)
     
-    def __submitUrlFinished(self):
+    def __submitUrlFinished(self, reply):
         """
         Private slot to determine the result of the URL scan submission.
+        
+        @param reply reference to the network reply
+        @type QNetworkReply
         """
-        reply = self.sender()
         if reply.error() == QNetworkReply.NoError:
             result = json.loads(str(reply.readAll(), "utf-8"))
             if result["response_code"] == self.ServiceResult_ItemPresent:
@@ -211,12 +216,14 @@ class VirusTotalAPI(QObject):
         reply.finished.connect(self.__getUrlScanReportUrlFinished)
         self.__replies.append(reply)
     
-    def __getUrlScanReportUrlFinished(self):
+    def __getUrlScanReportUrlFinished(self, reply):
         """
         Private slot to determine the result of the URL scan report URL
         request.
+        
+        @param reply reference to the network reply
+        @type QNetworkReply
         """
-        reply = self.sender()
         if reply.error() == QNetworkReply.NoError:
             result = json.loads(str(reply.readAll(), "utf-8"))
             if "filescan_id" in result and result["filescan_id"] is not None:
@@ -243,12 +250,14 @@ class VirusTotalAPI(QObject):
         reply.finished.connect(self.__getFileScanReportUrlFinished)
         self.__replies.append(reply)
     
-    def __getFileScanReportUrlFinished(self):
+    def __getFileScanReportUrlFinished(self, reply):
         """
         Private slot to determine the result of the file scan report URL
         request.
+        
+        @param reply reference to the network reply
+        @type QNetworkReply
         """
-        reply = self.sender()
         if reply.error() == QNetworkReply.NoError:
             result = json.loads(str(reply.readAll(), "utf-8"))
             self.fileScanReport.emit(result["permalink"])
@@ -284,11 +293,13 @@ class VirusTotalAPI(QObject):
         reply.finished.connect(self.__getIpAddressReportFinished)
         self.__replies.append(reply)
     
-    def __getIpAddressReportFinished(self):
+    def __getIpAddressReportFinished(self, reply):
         """
         Private slot to process the IP address report data.
+        
+        @param reply reference to the network reply
+        @type QNetworkReply
         """
-        reply = self.sender()
         if reply.error() == QNetworkReply.NoError:
             result = json.loads(str(reply.readAll(), "utf-8"))
             if result["response_code"] == 0:
@@ -343,14 +354,16 @@ class VirusTotalAPI(QObject):
         import Helpviewer.HelpWindow
         nam = Helpviewer.HelpWindow.HelpWindow.networkAccessManager()
         reply = nam.get(request)
-        reply.finished.connect(self.__getDomainReportFinished)
+        reply.finished.connect(lambda: self.__getDomainReportFinished(reply))
         self.__replies.append(reply)
     
-    def __getDomainReportFinished(self):
+    def __getDomainReportFinished(self, reply):
         """
         Private slot to process the IP address report data.
+        
+        @param reply reference to the network reply
+        @type QNetworkReply
         """
-        reply = self.sender()
         if reply.error() == QNetworkReply.NoError:
             result = json.loads(str(reply.readAll(), "utf-8"))
             if result["response_code"] == 0:
