@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2007 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2007 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -8,12 +8,17 @@ Module implementing a dialog to generate code for a Qt4/Qt5 dialog.
 """
 
 from __future__ import unicode_literals
+try:
+    str = unicode       # __IGNORE_EXCEPTION__
+except NameError:
+    pass
 
 import os
+import json
 import xml.etree.ElementTree
 
 from PyQt5.QtCore import QMetaObject, QByteArray, QRegExp, Qt, pyqtSlot, \
-    QMetaMethod, QSortFilterProxyModel
+    QMetaMethod, QSortFilterProxyModel, QProcess, QProcessEnvironment
 from PyQt5.QtGui import QStandardItemModel, QBrush, QStandardItem
 from PyQt5.QtWidgets import QWidget, QDialog, QDialogButtonBox, QAction
 from PyQt5 import uic
@@ -141,47 +146,155 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
         @return flag indicating an initialzation error (boolean)
         """
         return self.__initError
+    
+    def __runUicLoadUi(self, command):
+        """
+        Private method to run the UicLoadUi.py script with the given command
+        and return the output.
         
+        @param command uic command to be run
+        @type str
+        @return tuple of process output and error flag
+        @rtype tuple of (str, bool)
+        """
+        venvName = self.project.getDebugProperty("VIRTUALENV")
+        venvManager = e5App().getObject("VirtualEnvManager")
+        interpreter = venvManager.getVirtualenvInterpreter(venvName)
+        execPath = venvManager.getVirtualenvExecPath(venvName)
+        
+        env = QProcessEnvironment.systemEnvironment()
+        if execPath:
+            if env.contains("PATH"):
+                env.insert(
+                    "PATH", os.pathsep.join([execPath, env.value("PATH")])
+                )
+            else:
+                env.insert("PATH", execPath)
+        
+        loadUi = os.path.join(os.path.dirname(__file__), "UicLoadUi.py")
+        args = [
+            loadUi,
+            command,
+            self.formFile,
+            self.project.getProjectPath(),
+        ]
+        
+        uicText = ""
+        ok = False
+        
+        proc = QProcess()
+        proc.setWorkingDirectory(self.project.getProjectPath())
+        proc.setProcessEnvironment(env)
+        proc.start(interpreter, args)
+        started = proc.waitForStarted(5000)
+        finished = proc.waitForFinished(30000)
+        if started and finished:
+            output = proc.readAllStandardOutput()
+            outText = str(output, "utf-8", "replace")
+            if proc.exitCode() == 0:
+                ok = True
+                uicText = outText.strip()
+            else:
+                E5MessageBox.critical(
+                    self,
+                    self.tr("uic error"),
+                    self.tr(
+                        """<p>There was an error loading the form <b>{0}</b>"""
+                        """.</p><p>{1}</p>""").format(
+                        self.formFile, outText)
+                )
+        else:
+            E5MessageBox.critical(
+                self,
+                self.tr("uic error"),
+                self.tr(
+                    """<p>The project specific Python interpreter <b>{0}</b>"""
+                    """ could not be started or did not finish within 30"""
+                    """ seconds.</p>""").format(interpreter)
+            )
+        
+        return uicText, ok
+    
+    def __objectNameExternal(self):
+        """
+        Private method to get the object name of a form via an external
+        interpreter.
+        
+        @return object name
+        @rtype str
+        """
+        objectName = ""
+        
+        output, ok = self.__runUicLoadUi("object_name")
+        if ok and output:
+            objectName = output
+        
+        return objectName
+    
     def __objectName(self):
         """
-        Private method to get the object name of the dialog.
+        Private method to get the object name of a form.
         
-        @return object name (string)
+        @return object name
+        @rtype str
         """
-        try:
-            dlg = uic.loadUi(
-                self.formFile, package=self.project.getProjectPath())
-            return dlg.objectName()
-        except (AttributeError, ImportError,
-                xml.etree.ElementTree.ParseError) as err:
-            E5MessageBox.critical(
-                self,
-                self.tr("uic error"),
-                self.tr(
-                    """<p>There was an error loading the form <b>{0}</b>"""
-                    """.</p><p>{1}</p>""").format(self.formFile, str(err)))
-            return ""
+        if self.project.getDebugProperty("VIRTUALENV"):
+            return self.__objectNameExternal()
+        else:
+            try:
+                dlg = uic.loadUi(
+                    self.formFile, package=self.project.getProjectPath())
+                return dlg.objectName()
+            except (AttributeError, ImportError,
+                    xml.etree.ElementTree.ParseError) as err:
+                E5MessageBox.critical(
+                    self,
+                    self.tr("uic error"),
+                    self.tr(
+                        """<p>There was an error loading the form <b>{0}</b>"""
+                        """.</p><p>{1}</p>""").format(self.formFile, str(err)))
+                return ""
+    
+    def __classNameExternal(self):
+        """
+        Private method to get the class name of a form via an external
+        interpreter.
         
+        @return class name
+        @rtype str
+        """
+        className = ""
+        
+        output, ok = self.__runUicLoadUi("class_name")
+        if ok and output:
+            className = output
+        
+        return className
+    
     def __className(self):
         """
-        Private method to get the class name of the dialog.
+        Private method to get the class name of a form.
         
-        @return class name (sting)
+        @return class name
+        @rtype str
         """
-        try:
-            dlg = uic.loadUi(
-                self.formFile, package=self.project.getProjectPath())
-            return dlg.metaObject().className()
-        except (AttributeError, ImportError,
-                xml.etree.ElementTree.ParseError) as err:
-            E5MessageBox.critical(
-                self,
-                self.tr("uic error"),
-                self.tr(
-                    """<p>There was an error loading the form <b>{0}</b>"""
-                    """.</p><p>{1}</p>""").format(self.formFile, str(err)))
-            return ""
-        
+        if self.project.getDebugProperty("VIRTUALENV"):
+            return self.__classNameExternal()
+        else:
+            try:
+                dlg = uic.loadUi(
+                    self.formFile, package=self.project.getProjectPath())
+                return dlg.metaObject().className()
+            except (AttributeError, ImportError,
+                    xml.etree.ElementTree.ParseError) as err:
+                E5MessageBox.critical(
+                    self,
+                    self.tr("uic error"),
+                    self.tr(
+                        """<p>There was an error loading the form <b>{0}</b>"""
+                        """.</p><p>{1}</p>""").format(self.formFile, str(err)))
+                return ""
+    
     def __signatures(self):
         """
         Private slot to get the signatures.
@@ -216,148 +329,214 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
         """
         mapped = bytes(type_).decode()
         
+        # I. always check for *
+        mapped = mapped.replace("*", "")
+        
         if self.project.getProjectLanguage() != "Python2" or \
-           self.project.getProjectType == "PySide":
+           self.project.getProjectType in ("PySide", "PySide2"):
             # 1. check for const
             mapped = mapped.replace("const ", "")
             
-            # 2. check for *
-            mapped = mapped.replace("*", "")
-            
-            # 3. replace QString and QStringList
+            # 2. replace QString and QStringList
             mapped = mapped.replace("QStringList", "list")\
                            .replace("QString", "str")
             
-            # 4. replace double by float
+            # 3. replace double by float
             mapped = mapped.replace("double", "float")
         
         return mapped
-        
+    
+    def __updateSlotsModelExternal(self):
+        """
+        Private slot to update the slots tree display getting the data via an
+        external interpreter.
+        """
+        output, ok = self.__runUicLoadUi("signatures")
+        if ok and output:
+            objectsList = json.loads(output.strip())
+            
+            signatureList = self.__signatures()
+            
+            self.slotsModel.clear()
+            self.slotsModel.setHorizontalHeaderLabels([""])
+            for objectDict in objectsList:
+                itm = QStandardItem("{0} ({1})".format(
+                    objectDict["name"],
+                    objectDict["class_name"]))
+                self.slotsModel.appendRow(itm)
+                for methodDict in objectDict["methods"]:
+                    itm2 = QStandardItem(methodDict["signature"])
+                    itm.appendRow(itm2)
+                    
+                    if self.__module is not None:
+                        if methodDict["methods"][0] in signatureList or \
+                           methodDict["methods"][1] in signatureList:
+                            itm2.setFlags(
+                                Qt.ItemFlags(Qt.ItemIsEnabled))
+                            itm2.setCheckState(Qt.Checked)
+                            itm2.setForeground(QBrush(Qt.blue))
+                            continue
+            
+                    itm2.setData(methodDict["pyqt_signature"],
+                                 pyqtSignatureRole)
+                    itm2.setData(methodDict["python_signature"],
+                                 pythonSignatureRole)
+                    itm2.setData(methodDict["return_type"],
+                                 returnTypeRole)
+                    itm2.setData(methodDict["parameter_types"],
+                                 parameterTypesListRole)
+                    itm2.setData(methodDict["parameter_names"],
+                                 parameterNamesListRole)
+                    
+                    itm2.setFlags(Qt.ItemFlags(
+                        Qt.ItemIsUserCheckable |
+                        Qt.ItemIsEnabled |
+                        Qt.ItemIsSelectable)
+                    )
+                    itm2.setCheckState(Qt.Unchecked)
+            
+            self.slotsView.sortByColumn(0, Qt.AscendingOrder)
+    
     def __updateSlotsModel(self):
         """
         Private slot to update the slots tree display.
         """
         self.filterEdit.clear()
         
-        try:
-            dlg = uic.loadUi(
-                self.formFile, package=self.project.getProjectPath())
-            objects = dlg.findChildren(QWidget) + dlg.findChildren(QAction)
-            
-            signatureList = self.__signatures()
-            
-            self.slotsModel.clear()
-            self.slotsModel.setHorizontalHeaderLabels([""])
-            for obj in objects:
-                name = obj.objectName()
-                if not name or name.startswith("qt_"):
-                    # ignore un-named or internal objects
-                    continue
+        if self.project.getDebugProperty("VIRTUALENV"):
+            self.__updateSlotsModelExternal()
+        else:
+            try:
+                dlg = uic.loadUi(
+                    self.formFile, package=self.project.getProjectPath())
+                objects = dlg.findChildren(QWidget) + dlg.findChildren(QAction)
                 
-                metaObject = obj.metaObject()
-                className = metaObject.className()
-                itm = QStandardItem("{0} ({1})".format(name, className))
-                self.slotsModel.appendRow(itm)
-                for index in range(metaObject.methodCount()):
-                    metaMethod = metaObject.method(index)
-                    if metaMethod.methodType() == QMetaMethod.Signal:
-                        if qVersionTuple() >= (5, 0, 0):
-                            itm2 = QStandardItem("on_{0}_{1}".format(
-                                name,
-                                bytes(metaMethod.methodSignature()).decode()))
-                        else:
-                            itm2 = QStandardItem("on_{0}_{1}".format(
-                                name, metaMethod.signature()))
-                        itm.appendRow(itm2)
-                        if self.__module is not None:
+                signatureList = self.__signatures()
+                
+                self.slotsModel.clear()
+                self.slotsModel.setHorizontalHeaderLabels([""])
+                for obj in objects:
+                    name = obj.objectName()
+                    if not name or name.startswith("qt_"):
+                        # ignore un-named or internal objects
+                        continue
+                    
+                    metaObject = obj.metaObject()
+                    className = metaObject.className()
+                    itm = QStandardItem("{0} ({1})".format(name, className))
+                    self.slotsModel.appendRow(itm)
+                    for index in range(metaObject.methodCount()):
+                        metaMethod = metaObject.method(index)
+                        if metaMethod.methodType() == QMetaMethod.Signal:
                             if qVersionTuple() >= (5, 0, 0):
-                                method = "on_{0}_{1}".format(
+                                itm2 = QStandardItem("on_{0}_{1}".format(
                                     name,
                                     bytes(metaMethod.methodSignature())
-                                    .decode().split("(")[0])
+                                    .decode()
+                                ))
                             else:
-                                method = "on_{0}_{1}".format(
-                                    name, metaMethod.signature().split("(")[0])
-                            method2 = "{0}({1})".format(
-                                method, ", ".join(
-                                    [self.__mapType(t)
-                                     for t in metaMethod.parameterTypes()]))
-                            
-                            if method2 in signatureList or \
-                                    method in signatureList:
-                                itm2.setFlags(Qt.ItemFlags(Qt.ItemIsEnabled))
-                                itm2.setCheckState(Qt.Checked)
-                                itm2.setForeground(QBrush(Qt.blue))
-                                continue
-                        
-                        returnType = self.__mapType(
-                            metaMethod.typeName().encode())
-                        if returnType == 'void':
-                            returnType = ""
-                        parameterTypesList = [
-                            self.__mapType(t)
-                            for t in metaMethod.parameterTypes()]
-                        pyqtSignature = ", ".join(parameterTypesList)
-                        
-                        parameterNames = metaMethod.parameterNames()
-                        if parameterNames:
-                            for index in range(len(parameterNames)):
-                                if not parameterNames[index]:
-                                    parameterNames[index] = \
-                                        QByteArray("p{0:d}".format(index)
-                                                   .encode("utf-8"))
-                        parameterNamesList = [bytes(n).decode()
-                                              for n in parameterNames]
-                        methNamesSig = ", ".join(parameterNamesList)
-                        
-                        if methNamesSig:
-                            if qVersionTuple() >= (5, 0, 0):
-                                pythonSignature = \
-                                    "on_{0}_{1}(self, {2})".format(
+                                itm2 = QStandardItem("on_{0}_{1}".format(
+                                    name, metaMethod.signature()
+                                ))
+                            itm.appendRow(itm2)
+                            if self.__module is not None:
+                                if qVersionTuple() >= (5, 0, 0):
+                                    method = "on_{0}_{1}".format(
                                         name,
                                         bytes(metaMethod.methodSignature())
-                                        .decode().split("(")[0],
-                                        methNamesSig)
-                            else:
-                                pythonSignature = \
-                                    "on_{0}_{1}(self, {2})".format(
+                                        .decode().split("(")[0])
+                                else:
+                                    method = "on_{0}_{1}".format(
                                         name,
-                                        metaMethod.signature().split("(")[0],
-                                        methNamesSig)
-                        else:
-                            if qVersionTuple() >= (5, 0, 0):
-                                pythonSignature = "on_{0}_{1}(self)".format(
-                                    name,
-                                    bytes(metaMethod.methodSignature())
-                                    .decode().split("(")[0])
+                                        metaMethod.signature().split("(")[0])
+                                method2 = "{0}({1})".format(
+                                    method,
+                                    ", ".join([
+                                        self.__mapType(t)
+                                        for t in metaMethod.parameterTypes()
+                                    ])
+                                )
+                                
+                                if method2 in signatureList or \
+                                        method in signatureList:
+                                    itm2.setFlags(
+                                        Qt.ItemFlags(Qt.ItemIsEnabled))
+                                    itm2.setCheckState(Qt.Checked)
+                                    itm2.setForeground(QBrush(Qt.blue))
+                                    continue
+                            
+                            returnType = self.__mapType(
+                                metaMethod.typeName().encode())
+                            if returnType == 'void':
+                                returnType = ""
+                            parameterTypesList = [
+                                self.__mapType(t)
+                                for t in metaMethod.parameterTypes()]
+                            pyqtSignature = ", ".join(parameterTypesList)
+                            
+                            parameterNames = metaMethod.parameterNames()
+                            if parameterNames:
+                                for index in range(len(parameterNames)):
+                                    if not parameterNames[index]:
+                                        parameterNames[index] = \
+                                            QByteArray("p{0:d}".format(index)
+                                                       .encode("utf-8"))
+                            parameterNamesList = [bytes(n).decode()
+                                                  for n in parameterNames]
+                            methNamesSig = ", ".join(parameterNamesList)
+                            
+                            if methNamesSig:
+                                if qVersionTuple() >= (5, 0, 0):
+                                    pythonSignature = \
+                                        "on_{0}_{1}(self, {2})".format(
+                                            name,
+                                            bytes(metaMethod.methodSignature())
+                                            .decode().split("(")[0],
+                                            methNamesSig)
+                                else:
+                                    pythonSignature = \
+                                        "on_{0}_{1}(self, {2})".format(
+                                            name,
+                                            metaMethod.signature()
+                                            .split("(")[0],
+                                            methNamesSig)
                             else:
-                                pythonSignature = "on_{0}_{1}(self)".format(
-                                    name,
-                                    metaMethod.signature().split("(")[0])
-                        itm2.setData(pyqtSignature, pyqtSignatureRole)
-                        itm2.setData(pythonSignature, pythonSignatureRole)
-                        itm2.setData(returnType, returnTypeRole)
-                        itm2.setData(parameterTypesList,
-                                     parameterTypesListRole)
-                        itm2.setData(parameterNamesList,
-                                     parameterNamesListRole)
-                        
-                        itm2.setFlags(Qt.ItemFlags(
-                            Qt.ItemIsUserCheckable |
-                            Qt.ItemIsEnabled |
-                            Qt.ItemIsSelectable)
-                        )
-                        itm2.setCheckState(Qt.Unchecked)
-            
-            self.slotsView.sortByColumn(0, Qt.AscendingOrder)
-        except (AttributeError, ImportError,
-                xml.etree.ElementTree.ParseError) as err:
-            E5MessageBox.critical(
-                self,
-                self.tr("uic error"),
-                self.tr(
-                    """<p>There was an error loading the form <b>{0}</b>"""
-                    """.</p><p>{1}</p>""").format(self.formFile, str(err)))
+                                if qVersionTuple() >= (5, 0, 0):
+                                    pythonSignature = "on_{0}_{1}(self)"\
+                                        .format(
+                                            name,
+                                            bytes(metaMethod.methodSignature())
+                                            .decode().split("(")[0])
+                                else:
+                                    pythonSignature = "on_{0}_{1}(self)"\
+                                        .format(
+                                            name,
+                                            metaMethod.signature().split(
+                                                "(")[0])
+                            itm2.setData(pyqtSignature, pyqtSignatureRole)
+                            itm2.setData(pythonSignature, pythonSignatureRole)
+                            itm2.setData(returnType, returnTypeRole)
+                            itm2.setData(parameterTypesList,
+                                         parameterTypesListRole)
+                            itm2.setData(parameterNamesList,
+                                         parameterNamesListRole)
+                            
+                            itm2.setFlags(Qt.ItemFlags(
+                                Qt.ItemIsUserCheckable |
+                                Qt.ItemIsEnabled |
+                                Qt.ItemIsSelectable)
+                            )
+                            itm2.setCheckState(Qt.Unchecked)
+                
+                self.slotsView.sortByColumn(0, Qt.AscendingOrder)
+            except (AttributeError, ImportError,
+                    xml.etree.ElementTree.ParseError) as err:
+                E5MessageBox.critical(
+                    self,
+                    self.tr("uic error"),
+                    self.tr(
+                        """<p>There was an error loading the form <b>{0}</b>"""
+                        """.</p><p>{1}</p>""").format(self.formFile, str(err)))
         
     def __generateCode(self):
         """
@@ -396,6 +575,10 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                         tmplName = os.path.join(
                             getConfig('ericCodeTemplatesDir'),
                             "impl_pyside.py2.tmpl")
+                    elif self.project.getProjectType() == "PySide2":
+                        tmplName = os.path.join(
+                            getConfig('ericCodeTemplatesDir'),
+                            "impl_pyside2.py2.tmpl")
                     elif self.project.getProjectType() == "PyQt5":
                         tmplName = os.path.join(
                             getConfig('ericCodeTemplatesDir'),
@@ -409,6 +592,10 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                         tmplName = os.path.join(
                             getConfig('ericCodeTemplatesDir'),
                             "impl_pyside.py.tmpl")
+                    elif self.project.getProjectType() == "PySide2":
+                        tmplName = os.path.join(
+                            getConfig('ericCodeTemplatesDir'),
+                            "impl_pyside2.py.tmpl")
                     elif self.project.getProjectType() in [
                             "PyQt5", "E6Plugin"]:
                         tmplName = os.path.join(
@@ -487,14 +674,14 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
         
         # do the coding stuff
         if self.project.getProjectLanguage() == "Python2":
-            if self.project.getProjectType() == "PySide":
+            if self.project.getProjectType() in ("PySide", "PySide2"):
                 pyqtSignatureFormat = '@Slot({0})'
             elif self.project.getProjectType() == "PyQt5":
                 pyqtSignatureFormat = '@pyqtSlot({0})'
             else:
                 pyqtSignatureFormat = '@pyqtSignature("{0}")'
         else:
-            if self.project.getProjectType() == "PySide":
+            if self.project.getProjectType() in ("PySide", "PySide2"):
                 pyqtSignatureFormat = '@Slot({0})'
             else:
                 pyqtSignatureFormat = '@pyqtSlot({0})'
