@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -119,7 +119,7 @@ class SpellChecker(QObject):
         if isException:
             dicFile = Preferences.getEditor("SpellCheckingPersonalExcludeList")
             if not dicFile:
-                dicFile = SpellChecker.getDefaultPath(False)
+                dicFile = SpellChecker.getDefaultPath(True)
         else:
             dicFile = Preferences.getEditor("SpellCheckingPersonalWordList")
             if not dicFile:
@@ -129,7 +129,7 @@ class SpellChecker(QObject):
     @classmethod
     def _getDict(cls, lang, pwl="", pel=""):
         """
-        Protected classmethod to get a new dictionary.
+        Protected class method to get a new dictionary.
         
         @param lang the language to be used as the default (string).
             The string should be in language locale format (e.g. en_US, de).
@@ -195,7 +195,8 @@ class SpellChecker(QObject):
         
         @param size minimum word size (integer)
         """
-        self.minimumWordSize = size
+        if size > 0:
+            self.minimumWordSize = size
     
     def __getNextWord(self, pos, endPosition):
         """
