@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -23,7 +23,8 @@ class HistoryModel(QAbstractTableModel):
     UrlRole = Qt.UserRole + 3
     UrlStringRole = Qt.UserRole + 4
     TitleRole = Qt.UserRole + 5
-    MaxRole = TitleRole
+    VisitCountRole = Qt.UserRole + 6
+    MaxRole = VisitCountRole
     
     def __init__(self, historyManager, parent=None):
         """
@@ -40,6 +41,7 @@ class HistoryModel(QAbstractTableModel):
         self.__headers = [
             self.tr("Title"),
             self.tr("Address"),
+            self.tr("Visit Count")
         ]
         
         self.__historyManager.historyReset.connect(self.historyReset)
@@ -109,11 +111,15 @@ class HistoryModel(QAbstractTableModel):
             return itm.url
         elif role == self.TitleRole:
             return itm.userTitle()
+        elif role == self.VisitCountRole:
+            return itm.visitCount
         elif role in [Qt.DisplayRole, Qt.EditRole]:
             if index.column() == 0:
                 return itm.userTitle()
             elif index.column() == 1:
                 return itm.url
+            elif index.column() == 2:
+                return itm.visitCount
         elif role == Qt.DecorationRole:
             if index.column() == 0:
                 return WebBrowser.WebBrowserWindow.WebBrowserWindow.icon(
