@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2016 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2016 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -52,12 +52,14 @@ class WebHitTestResult(object):
                 if (!e)
                     return;
                 function isMediaElement(e) {{
-                    return e.tagName == 'AUDIO' || e.tagName == 'VIDEO';
+                    return e.tagName.toLowerCase() == 'audio' ||
+                           e.tagName.toLowerCase() == 'video';
                 }}
                 function isEditableElement(e) {{
                     if (e.isContentEditable)
                         return true;
-                    if (e.tagName == 'INPUT' || e.tagName == 'TEXTAREA')
+                    if (e.tagName.toLowerCase() == 'input' ||
+                        e.tagName.toLowerCase() == 'textarea')
                         return e.getAttribute('readonly') != 'readonly';
                     return false;
                 }}
@@ -83,16 +85,16 @@ class WebHitTestResult(object):
                 }};
                 var r = e.getBoundingClientRect();
                 res.boundingRect = [r.top, r.left, r.width, r.height];
-                if (e.tagName == 'IMG')
+                if (e.tagName.toLowerCase() == 'img')
                     res.imageUrl = e.getAttribute('src');
-                if (e.tagName == 'A') {{
+                if (e.tagName.toLowerCase() == 'a') {{
                     res.linkTitle = e.text;
                     res.linkUrl = e.getAttribute('href');
                 }}
                 while (e) {{
-                    if (res.linkTitle == '' && e.tagName == 'A')
+                    if (res.linkTitle == '' && e.tagName.toLowerCase() == 'a')
                         res.linkTitle = e.text;
-                    if (res.linkUrl == '' && e.tagName == 'A')
+                    if (res.linkUrl == '' && e.tagName.toLowerCase() == 'a')
                         res.linkUrl = e.getAttribute('href');
                     if (res.mediaUrl == '' && isMediaElement(e)) {{
                         res.mediaUrl = e.currentSrc;
