@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2010 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2010 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -560,6 +560,16 @@ class WebBrowserTabWidget(E5TabWidget):
         for index in range(self.count() - 1, -1, -1):
             self.closeBrowserAt(index, shutdown=shutdown)
     
+    def closeBrowserView(self, browser):
+        """
+        Public method to close the given browser.
+        
+        @param browser reference to the web browser view to be closed
+        @type WebBrowserView
+        """
+        index = self.indexOf(browser)
+        self.closeBrowserAt(index)
+    
     def closeBrowserAt(self, index, shutdown=False):
         """
         Public slot to close a browser based on its index.
@@ -1012,14 +1022,9 @@ class WebBrowserTabWidget(E5TabWidget):
                         """You have %n tab(s) open.""", "", self.count()),
                 modal=True,
                 parent=self)
-            if self.__mainWindow.fromEric:
-                quitButton = mb.addButton(
-                    self.tr("&Close"), E5MessageBox.AcceptRole)
-                quitButton.setIcon(UI.PixmapCache.getIcon("close.png"))
-            else:
-                quitButton = mb.addButton(
-                    self.tr("&Quit"), E5MessageBox.AcceptRole)
-                quitButton.setIcon(UI.PixmapCache.getIcon("exit.png"))
+            quitButton = mb.addButton(
+                self.tr("&Quit"), E5MessageBox.AcceptRole)
+            quitButton.setIcon(UI.PixmapCache.getIcon("exit.png"))
             closeTabButton = mb.addButton(
                 self.tr("C&lose Current Tab"), E5MessageBox.AcceptRole)
             closeTabButton.setIcon(UI.PixmapCache.getIcon("tabClose.png"))
@@ -1050,6 +1055,22 @@ class WebBrowserTabWidget(E5TabWidget):
         """
         return self.__stackedUrlBar.currentWidget()
     
+    def urlBarForView(self, view):
+        """
+        Public method to get a reference to the UrlBar associated with the
+        given view.
+        
+        @param view reference to the view to get the urlbar for
+        @type WebBrowserView
+        @return reference to the associated urlbar
+        @rtype UrlBar
+        """
+        for urlbar in self.__stackedUrlBar.urlBars():
+            if urlbar.browser() is view:
+                return urlbar
+        
+        return None
+    
     def __lineEditReturnPressed(self, edit):
         """
         Private slot to handle the entering of an URL.
