@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 
@@ -1752,6 +1752,8 @@ class WebBrowserView(QWebEngineView):
         """
         Private method to handle the loadStarted signal.
         """
+        # reset search
+        self.findText("")
         self.__isLoading = True
         self.__progress = 0
     
